package com.muyu;

import com.muyu.common.redis.service.RedisService;
import lombok.AllArgsConstructor;
import lombok.Data;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

/**
 * @author DongZl
 * @description: 负载测试
 * @Date 2024/4/12 下午5:13
 */
@SpringBootTest(classes = LoadCenterApplication.class)
public class LoadTest {

    @Autowired
    private RedisService redisService;

    @Test
    public void load(){
        // 初始化序列
        redisService.setCacheObject("cursor", 0);
        List<WorkGatewayNode> nodeIdList = new ArrayList<>(){{
            add(new WorkGatewayNode("work-gateway-node-A", 8));
            add(new WorkGatewayNode("work-gateway-node-B", 12));
            add(new WorkGatewayNode("work-gateway-node-C", 2));
            add(new WorkGatewayNode("work-gateway-node-D", 39));
            add(new WorkGatewayNode("work-gateway-node-E", 39));
        }};
        // 100
        List<String> loadNodeList = new ArrayList<>();

        long count = nodeIdList.stream().mapToInt(WorkGatewayNode::getWeight).sum();
        if (count < 100){
            List<WorkGatewayNode> list = nodeIdList.stream()
                    .sorted((o1, o2) -> o2.getWeight() - o1.getWeight())
                    .toList();
            int countWeight = 0;
            for (long i = count ; i < 100 ; i++) {
                WorkGatewayNode workGatewayNode = list.get(countWeight++ % list.size());
                workGatewayNode.setWeight(workGatewayNode.getWeight() + 1);
            }
        }
        whFor:while (true){
            for (WorkGatewayNode workGatewayNode : nodeIdList) {
                int weight = workGatewayNode.getWeight();
                if (weight > 0){
                    loadNodeList.add(
                            workGatewayNode.getNodeId()
                    );
                    workGatewayNode.setWeight(weight - 1);
                }
            }
            int sum = nodeIdList.stream()
                    .mapToInt(WorkGatewayNode::getWeight)
                    .sum();
            if (sum <= 0){
                break whFor;
            }
        }
        redisService.deleteObject("work:node:gateway");
        redisService.setCacheList("work:node:gateway", loadNodeList);
        CountDownLatch countDownLatch = new CountDownLatch(3000);

        new Thread(() -> {
            for (int i = 0 ; i < 1000 ; i++) {
                Long cursor = redisService.increment("cursor", 1L);
                String nodeId = redisService.getCacheListValue("work:node:gateway", cursor % 100);
                System.out.println(Thread.currentThread().getName() + "：" + cursor + "-----" + nodeId);
                stiNode.sti(nodeId);
                countDownLatch.countDown();
            }
        }).start();
        new Thread(() -> {
            for (int i = 0 ; i < 1000 ; i++) {
                Long cursor = redisService.increment("cursor", 1L);
                String nodeId = redisService.getCacheListValue("work:node:gateway", cursor % 100);
                System.out.println(Thread.currentThread().getName() + "：" + cursor + "-----" + nodeId);
                stiNode.sti(nodeId);
                countDownLatch.countDown();
            }
        }).start();
        new Thread(() -> {
            for (int i = 0 ; i < 1000 ; i++) {
                Long cursor = redisService.increment("cursor", 1L);
                String nodeId = redisService.getCacheListValue("work:node:gateway", cursor % 100);
                System.out.println(Thread.currentThread().getName() + "：" + cursor + "-----" + nodeId);
                stiNode.sti(nodeId);
                countDownLatch.countDown();
            }
        }).start();
        try {
            countDownLatch.await();
            stiNode.show();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
}

class stiNode{
    private static Map<String, Integer> stiNodeMap = new HashMap<>();

    public synchronized static void sti(String nodeId){
        Integer stiCount = stiNodeMap.getOrDefault(nodeId, 0);
        stiNodeMap.put(nodeId, stiCount + 1);
    }

    public static void show(){
        stiNodeMap.forEach((key,val) -> {
            System.out.println(key + "-----" + val);
        });
    }
}

@Data
@AllArgsConstructor
class WorkGatewayNode {

    private String nodeId;

    private int weight;
}
